#!/usr/bin/perl -w

# DOCUMENT:  Parses vmstat-output
# VERSION:   $Revision: 1.9 $
# AUTHOR:    M. Beranek <marcus@beranek.de>
# COPYRIGHT: M. Beranek http://www.beranek.de

use strict;
use Getopt::Std;

# CVS-Revision:
my $VERSION = sprintf "%d.%02d", q$Revision: 1.9 $ =~ /(\d+)/g;

my $rrdCommand  = "rrdtool update";

# Edit "$rrdTemplate", if your vmstat-output differs from mine:
#
# "procs -----------memory---------- ---swap-- -----io---- -system-- ----cpu----"
# " r  b   swpd   free   buff  cache   si   so    bi    bo   in   cs us sy id wa"
#
#my $rrdTemplate = " -t run:block:swpd:free:buff:cache:si:so:bi:bo:int:cont:user:sys:idle:wait";
my $rrdTemplate = " -t run:block:wait:swpd:free:buff:cache:si:so:bi:bo:int:cont:user:sys:idle";


# -------------------------------------------------------------------
# Config-Params:

# Current time:
my $timestamp = time();

# Interval:
my $interval = 60;

# RRD-Archive:
my $file = "/var/log/monitor/vm.rrd";

# Debug:
my $debugLevel = 10;


# -------------------------------------------------------------------
# Commandline-Parameters:
my %params;
getopts('d:i:f:hv', \%params);


# -------------------------------------------------------------------
# Help:
my $help = <<HELP

$0 [-f rrdfile] [-d timestamp] [-i interval] [-v] [-h]
  
  -f 'rrdfile'
      Specifies the filename of the RRD-archive.

  -d 'timestamp'
      UNIX-Timestamp in epoch-seconds.
      Same as 'date +\%s'.
      
  -i 'interval'
      Interval in seconds between two measurements
      (i.e. 'vmstat 60' has an interval of 60 seconds).
      
  -v  Print version and exit
  
  -h  Print this help.
  
  
HELP
;


# -------------------------------------------------------------------
# Timestamp:
if( exists($params{ 'd' }) ){
	# Timestamp:
	$timestamp = int( $params{ 'd' } );

}


# -------------------------------------------------------------------
# Help:
if( $params{ 'h' } ){
	print $help;
	exit;
}

# -------------------------------------------------------------------
# Version:
if( $params{ 'v' } ){
	print "$0 v. $VERSION";
	exit;
}


# -------------------------------------------------------------------
# Parameter:
if( $params{ 'f' } ){
	$file = $params{ 'f' };
}


# -------------------------------------------------------------------
# Interval:
if( $params{ 'i' } ){
	$interval = int( $params{ 'i' } );
}



# -------------------------------------------------------------------
# Main-Program:
MAIN: {
	
	# Count lines:
	my $line = 1;
	
	# Timestamp:
	my $stamp = $timestamp;
	
	# Read from STDIN:
	while( <STDIN> ){
		
		# remove spaces, replace with tabs:
		# s/ +/\t/g;
		
		# remove spaces, replace with colon:
		s/ +/:/g;
		
		# remove trailing colon:
		s/^://g;
		
		# Command to execute:
		my $command = "$rrdCommand $file $rrdTemplate $stamp:$_";
		
		# Debug:
		debugInfo( $command, 5 );
		
		# the first three lines of vmstat-output are
		# garbage (2 lines header + 1 line statistics
		# without measured data), so we print the output
		# after line 3:
		system $command if( $line > 3 );
		
		# Count:
		++ $line;
		
		# The timestamp given with '-d' belongs to the
		# first line with statistics. Line 1 and 2 are 
		# contain headers, so we start adding the 
		# interval after line 2.
		
		# Count timestamp:
		$stamp += $interval if( $line > 2 );
		
	}
}


# -------------------------------------------------------------------
# Output debug-message:
sub debugInfo {
	# Params:
	my ($msg, $level ) = @_;
	
	# Print:
	print "DEBUG: $msg\n" if( $level <= $debugLevel );
	
}




=pod

=head1 NAME

parse_vm_output.pl - parses vmstat-output and pushs data into RRD

=head1 SYNOPSIS

parse_vm_output.pl [-f rrdfile] [-d timestamp] [-i interval] [-v] [-h]

=head1 DESCRIPTION

=over

=item B<-f 'rrdfile'>

Specifies the filename of the RRD-archive.

=item B<-d 'timestamp'>

UNIX-Timestamp in epoch-seconds.
Same as 'date +\%s'.

=item B<-i 'interval'>

Interval in seconds between two measurements
(i.e. 'vmstat 60' has an interval of 60 seconds).

=item B<-v>

Print version and exit

=item B<-h>

Print this help.

=back

=head1 VERSION

$Revision $

=head1 LIMITATIONS

This program was tested with linux-kernel 2.6.18
and procps 3.2.7 only. There is not guarantee that
this program will run with other configurations.
Please consult your vmstat manpage and adopt the
parameters of "$rrdTemplate" at the beginning of 
this program.

=head1 BUGS

No bugs known at this time.

=head1 AUTHOR

Marcus Beranek E<lt>marcus@beranek,deE<gt>
http://www.beranek.de

=cut

